'program for performing Bayesian model averaging using the R package "BMA", available at:
' http://cran.r-project.org/web/packages/BMA/index.html

!debug = 0
if !debug = 0 then
	logmode +addin
endif

'check a workfile exists
%wfname = @wfname
if %wfname="" then
	@uiprompt("No workfile is open")
	stop
endif

%origsmpl = @pagesmpl
%smpl = @pagesmpl

!doGui = 1
%eq = ""
!method=1
%method = """LM - Linear models"" ""GLM - Generalized linear models"""
%or = "20"
'GLM options
!family = 1
%familylist = "gaussian binomial poisson Gamma inverse.gaussian quasi"
%familylist_str="Normal Binomial Poisson Gamma Inverse-Gaussian Quasi"
!link = 1
%linklist = "identity log logit probit cloglog inverse sqrt 1/mu^2"
%linklist_str="Identity Log Logit Probit ""Complementary Log-log"" Inverse SquareRoot 1/mu^2"
!var = 1
%varlist = "constant mu(1-mu) mu mu^2 mu^3"

if @len(%args)>0 then
	!doGui = 0
	%eq = %args
endif
if @len(@option(1))>0 then
	%temp = @equaloption("smpl")
	if @len(%temp)>0 then
		%smpl = %temp
	endif		
	%temp = @equaloption("method")
	if @len(%temp)>0 then
		if %temp = @upper("LM") then
			!method = 1
		endif
		if %temp = @upper("GLM") then
			!method = 2
		endif		
	endif
	%temp  = @equaloption("family")
	if @len(%temp)>0 then
		if %temp = "NORMAL" then
			!family = 1
		endif
		if %temp = "BINOMIAL" then
			!family = 2
		endif
		if %temp = "POISSON" then
			!family = 3
		endif	
		if %temp = "GAMMA" then
			!family = 4
		endif		
		if %temp = "IGAUSS" then
			!family = 5
		endif		
		if %temp = "QUASI" then
			!family = 6
		endif					
	endif
	%temp = @equaloption("link")
	if @len(%temp)>0 then
		if %temp = "IDENTITY" then
			!link = 1
		endif
		if %temp = "LOG" then
			!link = 2
		endif
		if %temp = "LOGIT" then
			!link = 3
		endif
		if %temp = "PROBIT" then
			!link = 4
		endif			
		if %temp = "CLOGLOG" then
			!link = 5
		endif			
		if %temp = "INVERSE" then
			!link = 6
		endif			
		if %temp = "SQRT" then
			!link = 7
		endif			
		if %temp = "BOX" then
			!link = 8
		endif				
	endif
	%temp = @equaloption("var")
	if @len(%temp)>0 then
		if %temp="CON" then
			!var =1
		endif
		if %temp="BIN" then
			!var =2
		endif	
		if %temp="MU" then
			!var =3
		endif		
		if %temp="MU2" then
			!var =4
		endif		
		if %temp="MU3" then
			!var =5
		endif		
	endif						
endif

if !doGui = 1 then
	!result = @uidialog("caption","Bayesian Model Averaging", _
	"edit",%eq,"Dependent variable followed by list of regressors", 200, _
	"list",!method,"Method", %method, _
	"edit",%or,"A number for excluding models in Occam's window (optional)", _
	"edit", %smpl, "Sample", 50, _
	"colbreak", _
	"text","Error distribution and link function for generalized linear models. Please make a selection only for GLM regression.", _
	"list",!family, "Family",%familylist_str, _
	"list",!link, "Link", %linklist_str, _
	"list",!var,"Quasi family variance (optional)", %varlist)
	if !result = -1 then
		stop
	endif
endif
if (!method=1 and !family>1) or (!method=1 and !link>1) then
	@uiprompt("Linear models does not require the family or link selection.")
endif

smpl {%smpl}
if @lasterrnum > 0 then
	@uiprompt("Invalid sample")
	stop
endif

%dep = @word(%eq,1)
%regs = @wmid(%eq, 2)
if @instr(%eq,"=")>0 then
	@uiprompt("Explicit equation specification is not allowed. Please enter the llist of variable names.")
	smpl {%origsmpl}
	stop
endif

%family = @word(%familylist,!family)
%link = @word(%linklist,!link)
%variance = @word(%varlist,!var)

!maxerr = @maxerrcount
'open up connection to R
if !debug = 1 then
	xopen(r, case=lower)   'open an R session
else
	xopen(r, case=lower, nolog)
endif
%str = @lasterrstr
if (@instr(%str,"StatConnectorSrv")>0) then 
	seterr "R is not installed, please check your EViews documentation for instructions on installing R."
	smpl {%origsmpl}
	stop
endif
setmaxerrs !maxerr+1

'install and load the BMA package in R
xrun "if (sum(.packages(TRUE)==""BMA"")==0) install.packages(""BMA"")"
xrun "library(BMA)"		

'put dependent variable
xput(rtype=vector, name=y) {%dep}
%X = ""
for !i=1 to @wcount(%regs)
	%reg = @word(%regs,!i)
	if @upper(%reg) = "C" then
	else
		%X = %X + %reg + " "			
	endif
next
xput(rtype=data.frame, name=x) {%X}

'perform Bayesian model averaging for Linear Models
if !method = 1 then
	%cmd = "bma<-bicreg(x, y, strict=FALSE, OR=" + %or +")"	
endif
if !method = 2 then
	if %family="quasi" then
		%cmd = "bma<-bic.glm(x, y, strict=FALSE, OR=" + %or + ", glm.family=" + %family + "(link=""" + %link +""", variance=""" + %variance + """))"
	else
		%cmd =  "bma<-bic.glm(x, y, strict=FALSE, OR=" + %or + ",glm.family=" + %family + "(link=""" + %link +"""))"
	endif
endif
xrun %cmd
if @lasterrnum>0 then 
	@uiprompt("R reported an error - possibly invalid specification of error distribution family or link function.")
	smpl {%origsmpl}
	stop
endif

%coefs = @getnextname("_coefs")
%se = @getnextname("_se")
%size = @getnextname("_size")
%bic = @getnextname("_bic")
%postsd = @getnextname("_postsd")
%postmean = @getnextname("_postmean")
%postprob = @getnextname("_postprob")
%probne = @getnextname("_probne")
if !method =1 then
	xget(name={%coefs},type=matrix) bma$ols
else
	xget(name={%coefs},type=matrix) bma$mle	
endif
xget(name={%se},type=matrix) bma$se	
xget(name={%postsd},type=vector) bma$postsd
xget(name={%postmean},type=vector) bma$postmean
xget(name={%bic},type=vector) bma$bic
xget(name={%postprob},type=vector) bma$postprob
if !method =1 then
	%r2 = @getnextname("_r2")
	xget(name={%r2},type=vector) bma$r2	
endif
xget(name={%size},type=vector) bma$size
xget(name={%probne},type=vector) bma$probne0

'--------------------------------------------------------------------------------------------------------Table Output
%sn = "BMA"
if @isobject(%sn) then
	%sn = @getnextname(%sn)
endif
spool {%sn}

%tn = @getnextname("_table")
!rows = 7+@wcount(%X)		
table(!rows,4) {%tn}					
{%tn}.setwidth(1) !rows
{%tn}.setwidth(1) 19
{%tn}.setwidth(2:4) 14
!rowcounter=1
{%tn}(!rowcounter,1) = "Bayesian Model Averaging"
!rowcounter = !rowcounter + 1	
{%tn}(!rowcounter,1) = "Dependent Variable: " + @upper(%dep)
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Method: " + @word(%method,!method)
!rowcounter = !rowcounter + 1	
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MI")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1		
%samplestr = "Sample: " + %smpl 
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1		
%obsstr = "Included observations: "  + @str(@obssmpl)
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1		
if !method = 2 then
	%familystr = "Family: " + @upper(@word(%familylist_str,!family))
	{%tn}(!rowcounter,1) = %familystr
	!rowcounter=!rowcounter+1		
	%linkstr = "Link: " + @upper(@word(%linklist_str,!link))
	{%tn}(!rowcounter,1) = %linkstr
	!rowcounter=!rowcounter+1	
	if !family=6 then
		%varstr = "Variance: " +@upper(@word(%varlist,!var))
		{%tn}(!rowcounter,1) = %varstr
		!rowcounter=!rowcounter+1					
	endif	
endif	
{%tn}.setjust(a1:a!rowcounter) left	
{%tn}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1		
'============================
{%tn}(!rowcounter,1) = "Variable"
{%tn}(!rowcounter,2) = "Post Mean"
{%tn}(!rowcounter,3) = "Post SD"
{%tn}(!rowcounter,4) = "P!=0 (in %)"
'============================
!rowcounter = !rowcounter+1		
{%tn}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1	
{%tn}(!rowcounter,1) = "C" 
{%tn}(!rowcounter,4) = @str(100, "f.2")
for !j=1 to @wcount(%X)		
	%reg = @word(%X,!j)
	{%tn}(!rowcounter+!j,1) = @upper(%reg)
	setcell({%tn},!rowcounter+!j,4,{%probne}(!j), "c", -7)	
next
	'{%tn}(!rowcounter+!j,4) = @str({%probne}(!j),"f.5")	
for !i=1 to @rows({%postsd})
	if {%postmean}(!i)==0 then
		{%tn}(!rowcounter+!i-1,2) = "-"				
		{%tn}(!rowcounter+!i-1,3) = "-"				
	else
		{%tn}(!rowcounter+!i-1,2) = {%postmean}(!i)	
		{%tn}(!rowcounter+!i-1,3) =  {%postsd}(!i)
	endif
next
!rowcounter = !rowcounter+@wcount(%X)+1
{%tn}.setlines(!rowcounter) +d	

'add table to spool
{%sn}.append(name="BMA") {%tn}
{%sn}.displayname BMA "Bayesian Model Averaging"


'Table 1: model comparison---------------------------------------------------------------------
%tn1 = @getnextname("_table")
!rows = 8+@wcount(%X)		
!cols = 1+@rows({%coefs})
table(!rows,!cols) {%tn1}					
{%tn1}.setwidth(1) !rows
{%tn1}.setwidth(1) 19
{%tn1}.setwidth(2:!cols) 14

!rowcounter=1
{%tn1}(!rowcounter,1) = "Parameter Estimates of Competing Models"
!rowcounter = !rowcounter + 1
{%tn1}(!rowcounter,1) = "No. for excluding models in Occam's window: " + %or
!rowcounter = !rowcounter + 1
{%tn1}(!rowcounter,1) = "Dependent Variable: " + @upper(%dep)
!rowcounter = !rowcounter + 1
{%tn1}.setjust(a1:a!rowcounter) left
{%tn1}.setlines(!rowcounter) +d		
!rowcounter = !rowcounter + 1
'============================
{%tn1}(!rowcounter,1) = "Variable"
for !i=1 to @rows({%coefs})
	{%tn1}(!rowcounter,!i+1) = "Model" + @str(!i)
next
'============================
!rowcounter = !rowcounter+1		
{%tn1}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1	
{%tn1}(!rowcounter,1) = "C" 
!temprow = !rowcounter+2
for !j=1 to @wcount(%X)		
	%reg = @word(%X,!j)
	{%tn1}(!temprow,1) = @upper(%reg)
	!temprow = !temprow+2	
next
'Coefs
!temprow = !rowcounter
for !j=1 to @columns({%coefs})
	for !i=1 to @rows({%coefs})
		if {%coefs}(!i,!j)==0 then
			{%tn1}(!temprow,!i+1) = "-"
		else
			{%tn1}(!temprow,!i+1) = {%coefs}(!i,!j)						
		endif	
	next	
	!temprow = !temprow+2
next
'SEs
!temprow=0
	!temprow = !rowcounter+1
for !j=1 to @columns({%se})
	for !i=1 to @rows({%coefs})
		if {%se}(!i,!j)  ==0 then
			{%tn1}(!temprow,!i+1) = ""		
		else
			{%tn1}(!temprow,!i+1) = "(" + @str({%se}(!i,!j),"f.5") + ")"								
		endif
	next
	!temprow = !temprow +2
next
'==================================	
!rowcounter = !rowcounter+@columns({%coefs})*2
{%tn1}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1
!temp = !rowcounter
{%tn1}(!rowcounter,1) = "No of Variables"
for !i=1 to @rows({%coefs})
	{%tn1}(!rowcounter,!i+1) = @str({%size}(!i)+1,"f.6")
next
!rowcounter = !rowcounter+1	
if !method=1 then
	{%tn1}(!rowcounter,1) = "R-Squared"	 
	for !i=1 to @rows({%coefs})
		{%tn1}(!rowcounter,!i+1) = {%r2}(!i)/100
	next
	!rowcounter = !rowcounter+1	
endif
{%tn1}(!rowcounter,1) = "Bayesian Info Criterion"
for !i=1 to @rows({%coefs})
	{%tn1}(!rowcounter,!i+1) = {%bic}(!i,1)
	{%tn1}(!rowcounter+1,!i+1) ={%postprob}(!i)	'Post Prob		
next
!rowcounter = !rowcounter+1
{%tn1}(!rowcounter,1) = "Posterior Probability"	
!rowcounter = !rowcounter+1
{%tn1}.setlines(!rowcounter) +d	
{%tn1}.setjust(a!temp:a!rowcounter) left	

{%sn}.append(name="Models") {%tn1}
{%sn}.displayname Models "Competing models"


'make output table -------------------------------------------
show {%sn}

d(noerr) {%tn} {%tn1} {%coefs} {%bic} {%postsd} {%postmean} {%postprob} {%se} {%size} {%probne}
if !method=1 then
	d(noerr) {%r2}
endif
	 
if !debug = 0 then
	xclose
endif
smpl {%origsmpl}

'
