'program for performing Bayesian model averaging using the R package "mlogitBMA", available at:
' http://cran.r-project.org/web/packages/mlogitBMA/index.html

!debug = 0
if !debug = 0 then
	logmode +addin
endif

'check a workfile exists
%wfname = @wfname
if %wfname="" then
	@uiprompt("No workfile is open")
	stop
endif

%origsmpl = @pagesmpl
%smpl = @pagesmpl

!doGui = 1
%res=""		'response variable
%same=""	'individual-specific variables
%diff=""		'alternative-specific variables
!C = 1 		'include alternative-specific constants 

%base="1"		'base choice index

!method = 1 	'model averaging method
%method = """Begg and Gray approximation"" ""Maximum likelihood estimation"""

'commands andn options
if @len(%args)>0 then
	!doGui = 0
	%res = %0
	!isExist = @wfind(%args,"@")
	if !isExist = 0 then
		%same = @wmid(%args,2)	
	else
		!pos = @instr(%args," @ ")
		%same = @mid(@left(%args,!pos),3)	 
		%diff = @mid(%args, !pos+3)	
	endif
endif
if @len(@option(1))>0 then
	%temp = @equaloption("smpl")
	if @len(%temp)>0 then
		%smpl = %temp
	endif		
	%temp = @equaloption("base")
	if @len(%temp)>0 then
		%base = %temp
	endif
	!method = @hasoption("mnl") + 1
	!C = @hasoption("exc")-1 
endif

if !doGui = 1 then
	!result = @uidialog("caption","Bayesian Model Averaging for Multinomial Logit Models", _
	"text","N individuals make their choice out of J alternatives.", _
	"edit",%res,"Dependent / Response variable",   _
	"edit", %same, "A list of individual-specific regressors", 100, _
	"edit", %diff, "A list of alternative-specific regressors", 100, _
	"colbreak", _
	"edit",%base,"Index of base alternative (optional)", _
	"check",!C,"Include alternative-specific intercepts (optional)", _
	"RADIO",!method,"Estimation method",%method, _
	"edit", %smpl, "Sample", 100)
	if !result = -1 then
		stop
	endif
endif
if @floor({%res})<>{%res} then 'not an integer
	@uiprompt(%res + " is not a multinomial response.")
	stop	
endif 
if @len(%same)==0 and @len(%diff)==0 then
	@uiprompt("Please enter a list of regressors.")
	stop
endif
smpl {%smpl}
if @lasterrnum > 0 then
	@uiprompt("Invalid sample")
	stop
endif

!maxerr = @maxerrcount
'open up connection to R
if !debug = 1 then
	xopen(type=r, case=lower)   'open an R session
 	statusline R step	
else
	xopen(type=r, case=lower, nolog)
endif
%str = @lasterrstr
if (@instr(%str,"StatConnectorSrv")>0) then 
	seterr "R is not installed, please check your EViews documentation for instructions on installing R."
	smpl {%origsmpl}
	stop
endif
setmaxerrs !maxerr+1

'install and load the BMA package in R
xrun "if (sum(.packages(TRUE)==""mlogitBMA"")==0) install.packages(""mlogitBMA"")"
xrun "library(mlogitBMA)"

'Inputs: f, data, logical str for including intercepts & lik ftn
%f = @lower(%res) + " ~ "
%data = @lower(%res) + " " 
if @len(%same)>0 then
	for !i=1 to @wcount(%same)
		%reg = @word(%same,!i)
		if @upper(%reg) = "C" then
		else
			%f = %f + @lower(%reg) + " + "
			%data = %data + @lower(%reg) + " "			
		endif
	next	
	%f = @left(%f,@len(%f)-3)	
else
	%f = %f + " 1 "	
endif
if @len(%diff)>0 then  
	%f = %f + " | "
	for !i=1 to @wcount(%diff)
		%reg = @word(%diff,!i)
		if @upper(%reg) = "C" then
		else
			%f = %f + @lower(%reg) + " + "			
			%data = %data + @lower(%reg) + " "			
		endif
	next		
	%f = @left(%f,@len(%f)-3)
endif
if !C = 0 then 
	%f = %f + "-1"
endif
%TF = "TRUE FALSE"

'perform Bayesian model averaging for multinomial logit models
xput(rtype=data.frame, name=data) {%data}	'toss the data
%cmd =  "bma<-bic.mlogit(" + @lower(%f) + ", data, approx =" + @upper(@word(%TF,!method)) + ", base.choice= " + %base + ", verbose = TRUE)"
xrun %cmd
if @lasterrnum>0 then 
	@uiprompt("R reported an error. Possibly change the estimation method.")
	smpl {%origsmpl}
	stop
endif
'the following two lines purposely reamains due to possible future request 
'%cmd =  "bma<-bic.mlogit(" + %f + ", data, approx =" + @upper(@word(%TF,!method)) + ", include.intercepts = " + @upper(@word(%TF,2-!C)) + ", verbose = TRUE)"
'%cmd1 = "est<-estimate.mlogit(" + @lower(%f) + ", data, method = """ + @word(%optstr,!opt) + """)" 
%cmd2 = "spec<-mnl.spec(" + @lower(%f) + ", data)"		
xrun %cmd2

'Outputs
'1. bic.mlogit outputs
%coefs = @getnextname("_coefs")
%se = @getnextname("_se")
%size = @getnextname("_size")
%bic = @getnextname("_bic")
%postsd = @getnextname("_postsd")
%postmean = @getnextname("_postmean")
%postprob = @getnextname("_postprob")
%probne = @getnextname("_probne")
%formula = @getnextname("_formula")
%label = @getnextname("_label")
xget(name={%coefs},type=matrix) bma$bic.glm$mle	
xget(name={%se},type=matrix) bma$bic.glm$se	
xget(name={%postsd},type=vector) bma$bic.glm$postsd
xget(name={%postmean},type=vector) bma$bic.glm$postmean
xget(name={%bic},type=vector) bma$bic.glm$bic
xget(name={%postprob},type=vector) bma$bic.glm$postprob
xget(name={%size},type=vector) bma$bic.glm$size
xget(name={%probne},type=vector) bma$bic.glm$probne0
xget(name={%label}, type=svector) bma$bic.glm$label
'2.  mnl.sepc output
%frequency = @getnextname("_frequency")
%choices = @getnextname("_choices")	
xget(name={%frequency}, type=vector) spec$frequency
xget(name={%choices}, type=vector) spec$choices 'vector of alternatives

'--------------------------------------------------------------------------------------------------------Table Output
'preliminaries for tables
!n_asc_intercepts = 0
!n_diff_regs = 0
!n_same_regs = 0
!n_asc_intercepts = @rows({%choices})-2
!n_diff_regs = @wcount(%diff)*(@rows({%choices})-1)
!n_same_regs = @wcount(%same)
!ntotal = 1 + !n_asc_intercepts + !n_diff_regs + !n_same_regs

'spool object
%sn = "BMAMLOGIT"
if @isobject(%sn) then
	%sn = @getnextname(%sn)
endif
spool {%sn}

'Table0: multinomial variable description
%tn0 = @getnextname("_table0")
table(6,@rows({%frequency})+1) {%tn0}
!rowcounter = 1
{%tn0}(!rowcounter,1) = "Dependent Variable: " + @upper(@word(%data,1))
!rowcounter = !rowcounter + 1
{%tn0}(!rowcounter,1) = "Base choice name: " + @str({%choices}({%base}))
!rowcounter = !rowcounter + 1
{%tn0}(!rowcounter,1) = "Base choice index: " + %base
{%tn0}.setjust(a1:a!rowcounter) left	
!rowcounter = !rowcounter + 1
{%tn0}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter + 1
{%tn0}(!rowcounter,1) = "Alternative"
{%tn0}(!rowcounter+1,1) = "Frequency"
for !i=1 to @rows({%choices})
	{%tn0}(!rowcounter,!i+1) = @str({%choices}(!i))
	{%tn0}(!rowcounter+1,!i+1)=@str({%frequency}(!i))
next
!rowcounter = !rowcounter + 2
{%tn0}.setlines(!rowcounter) +d	
{%sn}.append(name="Multinomial_res") {%tn0}
{%sn}.displayname Multinomial_res "Multinomial response"

'Table1: model averaging: highlighting on each variable
%tn = @getnextname("_table")
!rows = 6+@wcount(%data)		
table(!rows,4) {%tn}					
{%tn}.setwidth(1) !rows
{%tn}.setwidth(1) 19
{%tn}.setwidth(2:4) 14
!rowcounter = 1 
{%tn}(!rowcounter,1) = "Bayesian Model Averaging"
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Dependent Variable: " + @upper(@word(%data,1))
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Method: " + @word(%method,!method)
!rowcounter = !rowcounter + 1	
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MM")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1		
%samplestr = "Sample: " + %smpl 
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1		
%obsstr = "Included observations: "  + @str(@obssmpl)
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1		
{%tn}.setjust(a1:a!rowcounter) left	
{%tn}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1		
'============================
{%tn}(!rowcounter,1) = "Variable"
{%tn}(!rowcounter,2) = "Post Mean"
{%tn}(!rowcounter,3) = "Post SD"
{%tn}(!rowcounter,4) = "P!=0 (in %)"
'============================
!rowcounter = !rowcounter+1		
{%tn}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1	
{%tn}(!rowcounter,1) = "C" 
{%tn}(!rowcounter,4) = @str(100, "f.2") 'always 100 for intercepts
!temprow = !rowcounter
for !i=1 to !n_asc_intercepts
	{%tn}(!temprow+!i,1) = "C(" + @str({%choices}(2+!i)) + ")" 	
next
!temprow = !rowcounter + !n_asc_intercepts
for !i=1 to @wcount(%diff)
	%diff_i = @word(%diff,!i)
	for !j=1 to @rows({%choices})-1
		{%tn}(!temprow+(!i-1)*(@rows({%choices})-1)+!j,1) = @upper(%diff_i) + "(" + @str({%choices}(!j+1)) + ")"
	next
next
!temprow = !rowcounter + !n_asc_intercepts + !n_diff_regs
for !i=1 to @wcount(%same)
	%same_i = @word(%same,!i)
	{%tn}(!temprow+!i,1) = @upper(%same_i) 
next
for !j=1 to @rows({%probne})		
	setcell({%tn},!rowcounter+!j,4,{%probne}(!j), "c", -7)	
next
for !i=1 to @rows({%postsd})
	if {%postmean}(!i)==0 then
		{%tn}(!rowcounter+!i-1,2) = "-"				
		{%tn}(!rowcounter+!i-1,3) = "-"				
	else
		{%tn}(!rowcounter+!i-1,2) = {%postmean}(!i)	
		{%tn}(!rowcounter+!i-1,3) =  {%postsd}(!i)
	endif
next
!rowcounter = !rowcounter+@rows({%postmean})		
{%tn}.setlines(!rowcounter) +d	

'add table to spool
{%sn}.append(name="BMA") {%tn}
{%sn}.displayname BMA "Bayesian Model Averaging"

'Table2: model comparison (will answer which regressors should be included in the model)
%tn1 = @getnextname("_table1")
!rows = 8+@wcount(%X)		
!cols = 1+@rows({%coefs})
table(!rows,!cols) {%tn1}					
{%tn1}.setwidth(1) !rows
{%tn1}.setwidth(1) 19
{%tn1}.setwidth(2:!cols) 14
!rowcounter=1
{%tn1}(!rowcounter,1) = "Parameter Estimates for Competing Models"
!rowcounter = !rowcounter + 1
{%tn1}(!rowcounter,1) = "Dependent Variable: " + @upper(@word(%data,1))
!rowcounter = !rowcounter + 1
{%tn1}.setjust(a1:a!rowcounter) left	
{%tn1}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1		
'============================
{%tn1}(!rowcounter,1) = "Variable"
for !i=1 to @rows({%coefs})
	{%tn1}(!rowcounter,!i+1) = "Model" + @str(!i)
next
'============================
!rowcounter = !rowcounter+1		
{%tn1}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1	
{%tn1}(!rowcounter,1) = "C" 
!temprow = !rowcounter+2
for !i=1 to !n_asc_intercepts
	{%tn1}(!temprow,1) = "C(" + @str({%choices}(2+!i)) + ")" 	
	!temprow = !temprow+2
next
for !i=1 to @wcount(%diff)
	%diff_i = @word(%diff,!i)
	for !j=1 to @rows({%choices})-1
		{%tn1}(!temprow,1) = @upper(%diff_i) + "(" + @str({%choices}(!j+1)) + ")"
		!temprow = !temprow+2	
	next
next
'!temprow = !rowcounter + (!n_asc_intercepts + !n_diff_regs)*2
for !i=1 to @wcount(%same)
	%same_i = @word(%same,!i)
	{%tn1}(!temprow,1) = @upper(%same_i) 
	!temprow = !temprow+2
next
'Coefs
!temprow = !rowcounter
for !j=1 to @columns({%coefs})
	for !i=1 to @rows({%coefs})
		if {%coefs}(!i,!j)==0 then
			{%tn1}(!temprow,!i+1) = "-"
		else
			{%tn1}(!temprow,!i+1) = {%coefs}(!i,!j)						
		endif	
	next	
	!temprow = !temprow+2
next
'SEs
!temprow=0
	!temprow = !rowcounter+1
for !j=1 to @columns({%se})
	for !i=1 to @rows({%coefs})
		if {%se}(!i,!j)  ==0 then
			{%tn1}(!temprow,!i+1) = ""		
		else
			{%tn1}(!temprow,!i+1) = "(" + @str({%se}(!i,!j),"f.5") + ")"								
		endif
	next
	!temprow = !temprow +2
next
'==================================	
!rowcounter = !rowcounter+@columns({%coefs})*2
{%tn1}.setlines(!rowcounter) +d	
!rowcounter = !rowcounter+1
!temp = !rowcounter
{%tn1}(!rowcounter,1) = "No of Variables"
for !i=1 to @rows({%coefs})
	{%tn1}(!rowcounter,!i+1) = @str({%size}(!i)+1,"f.6")
next
!rowcounter = !rowcounter+1	
{%tn1}(!rowcounter,1) = "Bayesian Info Criterion"
for !i=1 to @rows({%coefs})
	{%tn1}(!rowcounter,!i+1) = {%bic}(!i,1)
	{%tn1}(!rowcounter+1,!i+1) ={%postprob}(!i)	'Post Prob		
next
!rowcounter = !rowcounter+1
{%tn1}(!rowcounter,1) = "Posterior Probability"	
!rowcounter = !rowcounter+1
{%tn1}.setlines(!rowcounter) +d	
{%tn1}.setjust(a!temp:a!rowcounter) left	

{%sn}.append(name="Models") {%tn1}
{%sn}.displayname Models "Competing Models"

 show {%sn}
 
 'delete values
d(noerr) {%tn1} {%tn0} {%tn} {%coefs} {%se} {%size} {%bic} {%postsd} {%postmean} {%postprob} {%probne} {%frequency} {%choices} {%label} 
if !debug = 0 then
	xclose
endif
smpl {%origsmpl}
